package com.n0tice.api.client;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.ByteArrayBody;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.joda.time.DateTimeUtils;
import org.joda.time.LocalDateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;
import org.scribe.builder.ServiceBuilder;
import org.scribe.model.OAuthRequest;
import org.scribe.model.Response;
import org.scribe.model.Token;
import org.scribe.model.Verb;
import org.scribe.oauth.OAuthService;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.base.Joiner;
import com.google.common.base.Strings;
import com.n0tice.api.client.exceptions.AuthorisationException;
import com.n0tice.api.client.exceptions.BadRequestException;
import com.n0tice.api.client.exceptions.MissingCredentialsExeception;
import com.n0tice.api.client.exceptions.N0ticeException;
import com.n0tice.api.client.exceptions.NotAllowedException;
import com.n0tice.api.client.exceptions.NotFoundException;
import com.n0tice.api.client.exceptions.ParsingException;
import com.n0tice.api.client.model.AccessToken;
import com.n0tice.api.client.model.Content;
import com.n0tice.api.client.model.FlagType;
import com.n0tice.api.client.model.Group;
import com.n0tice.api.client.model.HistoryItem;
import com.n0tice.api.client.model.MediaFile;
import com.n0tice.api.client.model.MediaInfo;
import com.n0tice.api.client.model.MediaType;
import com.n0tice.api.client.model.ModerationAction;
import com.n0tice.api.client.model.ModerationComplaint;
import com.n0tice.api.client.model.ModerationComplaintType;
import com.n0tice.api.client.model.ModerationState;
import com.n0tice.api.client.model.NewUserResponse;
import com.n0tice.api.client.model.Noticeboard;
import com.n0tice.api.client.model.NoticeboardResultSet;
import com.n0tice.api.client.model.NoticeboardSearchQuery;
import com.n0tice.api.client.model.Notification;
import com.n0tice.api.client.model.Reoccurence;
import com.n0tice.api.client.model.Repost;
import com.n0tice.api.client.model.ResultSet;
import com.n0tice.api.client.model.SearchQuery;
import com.n0tice.api.client.model.Update;
import com.n0tice.api.client.model.User;
import com.n0tice.api.client.model.VideoAttachment;
import com.n0tice.api.client.model.Vote;
import com.n0tice.api.client.oauth.N0ticeOauthApi;
import com.n0tice.api.client.parsers.ExifParser;
import com.n0tice.api.client.parsers.HistoryParser;
import com.n0tice.api.client.parsers.MediaInfoParser;
import com.n0tice.api.client.parsers.ModerationComplaintParser;
import com.n0tice.api.client.parsers.NoticeboardParser;
import com.n0tice.api.client.parsers.SearchParser;
import com.n0tice.api.client.parsers.UserParser;
import com.n0tice.api.client.urls.SearchUrlBuilder;
import com.n0tice.api.client.urls.UrlBuilder;
import com.n0tice.api.client.util.HttpFetcher;

public class N0ticeApi {
	
	private static final String MODERATION_USER = "moderationUser";

	private static Logger log = Logger.getLogger(N0ticeApi.class);
	
	private static final int HTTP_OK = 200;
	private static final int HTTP_BAD_REQUEST = 400;
	private static final int HTTP_UNAUTHORISED = 401;
	private static final int HTTP_FORBIDDEN = 403;
	private static final int HTTP_NOT_FOUND = 404;
	
	private static final String UTF_8 = "UTF-8";
	private static final String COMMA = ",";		
	private static final DateTimeFormatter LOCAL_DATE_TIME_FORMAT = DateTimeFormat.forPattern("yyyy-MM-dd HH:mm");
	private static final DateTimeFormatter ZULE_TIME_FORMAT = ISODateTimeFormat.dateTimeNoMillis();
	
	private final String apiUrl;	
	private final UrlBuilder urlBuilder;
	private final SearchUrlBuilder searchUrlBuilder;
	private final HttpFetcher httpFetcher;
	private final SearchParser searchParser;
	private final UserParser userParser;
	private final NoticeboardParser noticeboardParser;
	private final ModerationComplaintParser moderationComplaintParser;
	private final HistoryParser historyParser;
	private final ExifParser exifParser;
	private final MediaInfoParser mediaInfoParser;

	private OAuthService service;
	private Token scribeAccessToken;
	
	public N0ticeApi(String apiUrl) {
		this.apiUrl = apiUrl;
		this.urlBuilder = new UrlBuilder(apiUrl);
		this.searchUrlBuilder = new SearchUrlBuilder(apiUrl);
		this.httpFetcher = new HttpFetcher();
		this.searchParser = new SearchParser();
		this.userParser = new UserParser();
		this.noticeboardParser = new NoticeboardParser();
		this.historyParser = new HistoryParser();
		this.moderationComplaintParser = new ModerationComplaintParser();
		this.exifParser = new ExifParser();
		this.mediaInfoParser = new MediaInfoParser();
	}
	
	public N0ticeApi(String apiUrl, String consumerKey, String consumerSecret, AccessToken accessToken) {
		this.apiUrl = apiUrl;
		this.urlBuilder = new UrlBuilder(apiUrl);
		this.searchUrlBuilder = new SearchUrlBuilder(apiUrl);
		this.httpFetcher = new HttpFetcher();
		this.searchParser = new SearchParser();
		this.userParser = new UserParser();
		this.noticeboardParser = new NoticeboardParser();
		this.historyParser = new HistoryParser();
		this.moderationComplaintParser = new ModerationComplaintParser();
		this.exifParser = new ExifParser();
		this.mediaInfoParser = new MediaInfoParser();
		
		service = new ServiceBuilder().provider(new N0ticeOauthApi(apiUrl))
			.apiKey(consumerKey)
			.apiSecret(consumerSecret)
			.build();
		scribeAccessToken = new Token(accessToken.getToken(), accessToken.getSecret());
	}
	
	public Content get(String id) throws N0ticeException {
		return searchParser.parseContent(httpFetcher.fetchContent(urlBuilder.get(id), UTF_8));
	}
	
	public List<HistoryItem> getHistory(String id) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.getHistory(id));
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == 200) {
			return historyParser.parse(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));		
	}
	
	public List<ModerationComplaint> getModerationComplaints(String id) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.get(id) + "/flags");
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == 200) {
			return moderationComplaintParser.parse(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));		
	}
	
	public void closeModerationComplaint(String contentId, int flagId, String moderationUser) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.POST, urlBuilder.closeModerationComplaint(contentId, flagId));
		addBodyParameter(request, MODERATION_USER, moderationUser);
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == 200) {
			getResponseBody(response);
			return;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));		
	}
	
	public void closeModerationComplaint(String contentId, int flagId) throws N0ticeException {
		closeModerationComplaint(contentId, flagId, null);
	}
	
	public List<Repost> getReposts(String id) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.get(id) + "/reposts");
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return searchParser.parseReposts(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));		
	}
	
	public Update getUpdate(String id) throws N0ticeException {
		return searchParser.parseUpdate(httpFetcher.fetchContent(urlBuilder.get(id), UTF_8));
	}
	
	public Content authedGet(String id) throws N0ticeException {		
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.get(id));
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return searchParser.parseContent(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Map<String, Map<String, String>> imageExif(String id) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.get("image/" + id + "/exif"));
		oauthSignRequest(request);

		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return exifParser.parse(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public MediaInfo videoMediaInfo(String id) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.get(id + "/mediainfo"));
		oauthSignRequest(request);

		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return mediaInfoParser.parse(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public ResultSet search(SearchQuery searchQuery) throws N0ticeException {
		return searchParser.parseSearchResults(httpFetcher.fetchContent(searchUrlBuilder.toUrl(searchQuery), UTF_8));
	}
	
	public NoticeboardResultSet searchNoticeboards(NoticeboardSearchQuery noticeboardSearchQuery) throws N0ticeException {
		return noticeboardParser.parseNoticeboardSearchResults(httpFetcher.fetchContent(searchUrlBuilder.toUrl(noticeboardSearchQuery), UTF_8));
	}
	
	public User user(String username) throws N0ticeException {
		return userParser.parseUserProfile(httpFetcher.fetchContent(urlBuilder.userProfile(username), UTF_8));
	}
	
	public List<User> followedUsers(String username) throws N0ticeException {
		return userParser.parseUserProfiles(httpFetcher.fetchContent(urlBuilder.userFollowedUsers(username), UTF_8));
	}
	
	public List<Noticeboard> followedNoticeboards(String username) throws N0ticeException {
		return noticeboardParser.parseNoticeboards(httpFetcher.fetchContent(urlBuilder.userFollowedNoticeboards(username), UTF_8));
	}
	
	public List<Noticeboard> noticeboards(String username) throws N0ticeException {
		return noticeboardParser.parseNoticeboards(httpFetcher.fetchContent(urlBuilder.userNoticeboards(username), UTF_8));
	}
	
	public Noticeboard noticeBoard(String noticeboard) throws N0ticeException {
		return noticeboardParser.parseNoticeboardResult(httpFetcher.fetchContent(urlBuilder.noticeBoard(noticeboard), UTF_8));
	}
	
	public User verify() throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/verify");	// TODO most be a POST
		oauthSignRequest(request);
		
		Response response = request.send();
		if (response.getCode() == HTTP_OK) {
	    	return userParser.parseUserProfile(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Content postReport(String headline, Double latitude, Double longitude, String body, String link, MediaFile image, VideoAttachment video, String noticeboard) throws N0ticeException {
		return postReport(headline, latitude, longitude, body, link, image, video, noticeboard, null);		
	}
	
	public Noticeboard createNoticeboard(String name, String description, boolean moderated, Date endDate, Date embargoDate, Set<MediaType> supportedMediaTypes, String group, 
			MediaFile cover, boolean featured, boolean isolated) throws N0ticeException {
		return postNewNoticeboard(null, name, description, moderated, endDate, embargoDate, supportedMediaTypes, group, cover, featured, isolated);
	}
	
	public Noticeboard createNoticeboard(String domain, String name, String description, boolean moderated, Date endDate, Date embargoDate, Set<MediaType> supportedMediaTypes, String group, MediaFile cover, boolean featured, boolean isolated) throws N0ticeException {
		return postNewNoticeboard(domain, name, description, moderated, endDate, embargoDate, supportedMediaTypes, group, cover, featured, isolated);
	}
	
	public Noticeboard editNoticeboard(String domain, String name, String description, Boolean moderated, Boolean featured, MediaFile cover, Date endDate, Date embargoDate, Set<MediaType> supportedMediaTypes, String group, Boolean isolated) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, urlBuilder.noticeBoard(domain));
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		
		addEntityPartParameter(entity, "name", name);
		addEntityPartParameter(entity, "description", description);
		addEntityPartParameter(entity, "moderated", moderated != null ? Boolean.toString(moderated) : null);
		addEntityPartParameter(entity, "featured", featured != null ? Boolean.toString(featured) : null);
		addEntityPartParameter(entity, "isolated", isolated != null ? Boolean.toString(isolated) : null);
		
		if (cover != null) {
			entity.addPart("cover", new ByteArrayBody(cover.getData(), cover.getFilename()));
		}
		if (endDate != null) {
			addEntityPartParameter(entity, "endDate", ISODateTimeFormat.dateTimeNoMillis().print(new DateTime(endDate)));
		}
		if (embargoDate != null) {
			addEntityPartParameter(entity, "embargoDate", ISODateTimeFormat.dateTimeNoMillis().print(new DateTime(embargoDate)));
		}
		if (supportedMediaTypes != null) {
			addEntityPartParameter(entity, "supportedMediaTypes", formatSupportMediaTypes(supportedMediaTypes).toString());
		}
		if (group != null) {
			addEntityPartParameter(entity, "group", group);
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
	    	return noticeboardParser.parseNoticeboardResult(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
 	
	public void closeNoticeboard(String domain) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, urlBuilder.closeNoticeboard(domain));
		oauthSignRequest(request);
		
		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Group createGroup(String name, String colour) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/groups/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		addEntityPartParameter(entity, "name", name);
		addEntityPartParameter(entity, "colour", colour);

		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseGroup(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	

	public Group createGroup(String name, String colour, String username) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/groups/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		addEntityPartParameter(entity, "name", name);
		addEntityPartParameter(entity, "colour", colour);
		addEntityPartParameter(entity, "username", username);
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseGroup(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Group group(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/" + id);
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseGroup(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public List<Group> groups(String username) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/user/" + username + "/groups");

		Response response = request.send();

		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseGroups(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Group updateGroup(String id, String name, String colour) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id);
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		if (!Strings.isNullOrEmpty(name)) {
			addStringPart(entity, "name", name);
		}
		if (!Strings.isNullOrEmpty(colour)) {
			addStringPart(entity, "colour", colour);
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseGroup(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Content postReport(String headline, Double latitude, Double longitude, String body, String link, MediaFile image, VideoAttachment video, String noticeboard, DateTime date) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/report/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		if (headline != null) {
			addStringPart(entity, "headline", headline);
		}
		if (noticeboard != null) {
			addStringPart(entity, "noticeboard", noticeboard);
		}

		if (latitude != null && longitude != null) {			
			addStringPart(entity, "latitude", Double.toString(latitude));
			addStringPart(entity, "longitude", Double.toString(longitude));						
		}
		
		populateUpdateFields(body, link, image, video, entity);
		
		if (date != null) {
			addStringPart(entity, "date", date.toString(ZULE_TIME_FORMAT));
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseContent(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public ResultSet authedSearch(SearchQuery searchQuery) throws N0ticeException {				
		OAuthRequest request = createOauthRequest(Verb.GET, searchUrlBuilder.toUrl(searchQuery));
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseSearchResults(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Content postEvent(String headline, double latitude,
			double longitude, String body, String link, MediaFile image, VideoAttachment video,
			String noticeboard, LocalDateTime startDate, LocalDateTime endDate, Reoccurence reoccurence, LocalDateTime reoccursTo) throws N0ticeException {		
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/event/new");

		final MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		addEntityPartParameter(entity, "headline", headline);
		addEntityPartParameter(entity, "noticeboard", noticeboard);
		addEntityPartParameter(entity, "latitude", Double.toString(latitude));
		addEntityPartParameter(entity, "longitude", Double.toString(longitude));
		populateUpdateFields(body, link, image, video, entity);
		
		addEntityPartParameter(entity, "startDate", startDate.toString(LOCAL_DATE_TIME_FORMAT));
		addEntityPartParameter(entity, "endDate", endDate.toString(LOCAL_DATE_TIME_FORMAT));
		if (reoccurence != null && reoccursTo != null) {
			addEntityPartParameter(entity, "reoccurs", reoccurence.toString());
			addEntityPartParameter(entity, "reoccursTo", reoccursTo.toString(LOCAL_DATE_TIME_FORMAT));
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseContent(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Content postReportUpdate(String reportId, String body, String link, MediaFile image, VideoAttachment video) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + reportId  + "/update/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		populateUpdateFields(body, link, image, video, entity);

		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseContent(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public void deleteUpdate(String updateId) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.DELETE, apiUrl + "/" + updateId);
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Update postEventUpdate(String eventId, String body, String link, MediaFile image, VideoAttachment video) throws N0ticeException {	// TODO could potentially be merged with above when all content types support updates
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + eventId  + "/update/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		populateUpdateFields(body, link, image, video, entity);
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseUpdate(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean voteInteresting(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id + "/vote/interesting");	
		oauthSignRequest(request);
		
		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}

		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean removeInterestingVote(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.DELETE, apiUrl + "/" + id + "/vote/interesting");	
		oauthSignRequest(request);
		
		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}

		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
		
	public boolean flagAsInappropriate(String id, ModerationComplaintType type, String notes, String email, String moderationUser) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id + "/flag");
		if (type != null) {
			addBodyParameter(request, "type", type.toString());
		}
		if (notes != null) {
			addBodyParameter(request, "notes", notes);
		}
		if (email != null) {
			addBodyParameter(request, "email", email);
		}
		addBodyParameter(request, MODERATION_USER, moderationUser);
		if (scribeAccessToken != null) {
			oauthSignRequest(request);
		}
		
		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean flagAsInappropriate(String id, ModerationComplaintType type, String notes, String email) throws N0ticeException {
		return flagAsInappropriate(id, type, notes, email, null);
	}
	
	public boolean repost(String id, String noticeboard) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id + "/repost");
		addBodyParameter(request, "noticeboard", noticeboard);
		oauthSignRequest(request);

		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}

		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public List<ModerationAction> moderationActions(String username) throws N0ticeException {
		return moderationComplaintParser.parseModerationActions(httpFetcher.fetchContent(urlBuilder.moderationActions(username), UTF_8));
	}
	
	public List<ModerationState> moderationStates(String username) throws N0ticeException {
		return moderationComplaintParser.parseModerationStates(httpFetcher.fetchContent(urlBuilder.moderationStates(username), UTF_8));
	}
	
	public List<FlagType> flagTypes() throws N0ticeException {
		return moderationComplaintParser.parseFlagTypes(httpFetcher.fetchContent(urlBuilder.flagTypes(), UTF_8));
	}
	
	public boolean moderate(String id, String notes, String action, String moderationUser) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id + "/moderate");
		addBodyParameter(request, "action", action);
		addBodyParameter(request, "notes", notes);
		addBodyParameter(request, MODERATION_USER, moderationUser);
		oauthSignRequest(request);
		
		final Response response = request.send();		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}

		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean moderate(String id, String notes, String action) throws N0ticeException {
		return moderate(id, notes, action, null);
	}
	
	public List<Vote> interestingVotes(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/" + id + "/votes/interesting");	
			
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			return searchParser.parseVotes(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public List<Notification> notifications(String username) throws N0ticeException {		
		OAuthRequest request = createOauthRequest(Verb.GET, urlBuilder.userNotifications(username));
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			return searchParser.parseNotifications(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Content updateReport(String id, String headline, String body, String noticeboard) throws N0ticeException {	
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/" + id);	
		addBodyParameter(request, "headline", headline);
		addBodyParameter(request, "body", body);
		addBodyParameter(request, "noticeboard", noticeboard);
		oauthSignRequest(request);
		
		Response response = request.send();
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseContent(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean followUser(String username) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/" + username + "/follow");	
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean unfollowUser(String username) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/" + username + "/unfollow");	
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean followNoticeboard(String noticeboard) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/noticeboard/" + noticeboard + "/follow");	
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean unfollowNoticeboard(String noticeboard) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/noticeboard/" + noticeboard + "/unfollow");	
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
	    	return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public NewUserResponse createUser(String consumerKey, String username, String password, String email) throws N0ticeException {
		return createUser(consumerKey, username, password, email, null, null, null);
	}
	
	public NewUserResponse createUser(String consumerKey, String username, String password, String email, String twitterAccessToken, String twitterAccessSecret, String facebookAccessToken) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/new");		
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "username", username);
		addBodyParameter(request, "password", password);
		addBodyParameter(request, "email", email);
		addBodyParameter(request, "twitterAccessToken", twitterAccessToken);
		addBodyParameter(request, "twitterAccessSecret", twitterAccessSecret);
		addBodyParameter(request, "facebookAccessToken", facebookAccessToken);
		
		final Response response = request.send();

		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseNewUserResponse(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public AccessToken authUser(String consumerKey, String username, String password, String consumerSecret) throws N0ticeException {
		log.info("Attempting to auth user: " + consumerKey + ", " + username + ", " + password + ", " + consumerSecret);
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/auth");
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "username", username);
		addBodyParameter(request, "password", password);

		// Manually sign this request using the consumer secret rather than the access key/access secret.
		addBodyParameter(request, "oauth_signature_method", "HMAC-SHA1");
		addBodyParameter(request, "oauth_version", "1.0");
		addBodyParameter(request, "oauth_timestamp", Long.toString(DateTimeUtils.currentTimeMillis()));
		final String effectiveUrl = request.getCompleteUrl() + "?" + request.getBodyContents();
		addBodyParameter(request, "oauth_signature", sign(effectiveUrl, consumerSecret));
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseAuthUserResponse(getResponseBody(response)).getAccessToken();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public AccessToken authGuardianUser(String consumerKey, String token, String consumerSecret) throws N0ticeException {
		log.info("Attempting to auth guardian user: " + consumerKey + ", " + token);
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/auth");
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "guardianToken", token);

		// Manually sign this request using the consumer secret rather than the access key/access secret.
		addBodyParameter(request, "oauth_signature_method", "HMAC-SHA1");
		addBodyParameter(request, "oauth_version", "1.0");
		addBodyParameter(request, "oauth_timestamp", Long.toString(DateTimeUtils.currentTimeMillis()));
		final String effectiveUrl = request.getCompleteUrl() + "?" + request.getBodyContents();
		addBodyParameter(request, "oauth_signature", sign(effectiveUrl, consumerSecret));
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseAuthUserResponse(getResponseBody(response)).getAccessToken();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public AccessToken authGuardianCookie(String consumerKey, String cookie, String consumerSecret) throws N0ticeException {
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/auth");
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "guardianCookie", cookie);

		// Manually sign this request using the consumer secret rather than the access key/access secret.
		addBodyParameter(request, "oauth_signature_method", "HMAC-SHA1");
		addBodyParameter(request, "oauth_version", "1.0");
		addBodyParameter(request, "oauth_timestamp", Long.toString(DateTimeUtils.currentTimeMillis()));
		final String effectiveUrl = request.getCompleteUrl() + "?" + request.getBodyContents();
		addBodyParameter(request, "oauth_signature", sign(effectiveUrl, consumerSecret));
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseAuthUserResponse(getResponseBody(response)).getAccessToken();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public AccessToken authTwitterUser(String consumerKey, String twitterAccessToken, String twitterAccessSecret, String consumerSecret) throws N0ticeException {
		return  authTwitterUser(consumerKey, twitterAccessToken, twitterAccessSecret, consumerSecret, true);
	}
	
	public AccessToken authTwitterUser(String consumerKey, String twitterAccessToken, String twitterAccessSecret, String consumerSecret, boolean createAccount) throws N0ticeException {
		log.info("Attempting to auth twitter user using twitter token: " + twitterAccessToken);
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/auth");
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "twitterAccessToken", twitterAccessToken);
		addBodyParameter(request, "twitterAccessSecret", twitterAccessSecret);
		addBodyParameter(request, "create", Boolean.toString(createAccount));

		// Manually sign this request using the consumer secret rather than the access key/access secret.
		addBodyParameter(request, "oauth_signature_method", "HMAC-SHA1");
		addBodyParameter(request, "oauth_version", "1.0");
		addBodyParameter(request, "oauth_timestamp", Long.toString(DateTimeUtils.currentTimeMillis()));
		final String effectiveUrl = request.getCompleteUrl() + "?" + request.getBodyContents();
		addBodyParameter(request, "oauth_signature", sign(effectiveUrl, consumerSecret));
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseAuthUserResponse(getResponseBody(response)).getAccessToken();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public AccessToken authFacebookUser(String consumerKey, String facebookToken, String consumerSecret) throws N0ticeException {
		return authFacebookUser(consumerKey, facebookToken, consumerSecret, true);
	}
	
	public AccessToken authFacebookUser(String consumerKey, String facebookToken, String consumerSecret, boolean createAccount) throws N0ticeException {
		log.info("Attempting to auth facebook user using facebook token: " + facebookToken);
		final OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/auth");
		addBodyParameter(request, "consumerkey", consumerKey);
		addBodyParameter(request, "facebookAccessToken", facebookToken);
		addBodyParameter(request, "create", Boolean.toString(createAccount));

		// Manually sign this request using the consumer secret rather than the access key/access secret.
		addBodyParameter(request, "oauth_signature_method", "HMAC-SHA1");
		addBodyParameter(request, "oauth_version", "1.0");
		addBodyParameter(request, "oauth_timestamp", Long.toString(DateTimeUtils.currentTimeMillis()));
		final String effectiveUrl = request.getCompleteUrl() + "?" + request.getBodyContents();
		addBodyParameter(request, "oauth_signature", sign(effectiveUrl, consumerSecret));
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {		
			return new UserParser().parseAuthUserResponse(getResponseBody(response)).getAccessToken();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public User updateUserDetails(String username, String displayName, String bio, MediaFile image, VideoAttachment preroll) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/user/" + username);		
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		if (displayName != null) {
			addStringPart(entity, "displayName", displayName);
		}
		if (bio != null) {
			addStringPart(entity, "bio", bio);
		}
		if (image != null) {
			entity.addPart("image", new ByteArrayBody(image.getData(), image.getFilename()));
		}
		if (preroll != null) {
			entity.addPart("preroll", new InputStreamBody(preroll.getData(), preroll.getFilename()));			
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();

		if (response.getCode() == HTTP_OK) {
			return new UserParser().parseUserProfile(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean deleteReport(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.DELETE, apiUrl + "/" + id);	
		oauthSignRequest(request);
		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
			getResponseBody(response);
			return true;
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public InputStream downloadVideo(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/" + id + "/original");	
		oauthSignRequest(request);

		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return response.getStream();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public InputStream downloadVideoPreview(String id) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/" + id + "/preview");	
		oauthSignRequest(request);
		
		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			// TODO need to capture Content-Length so that it can be relayed to the final end user.
			return response.getStream();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}

	public InputStream previewImage(String id, String size) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/" + id + "/preview/" + size);	
		oauthSignRequest(request);

		final Response response = request.send();
		if (response.getCode() == HTTP_OK) {
			return response.getStream();
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Map<String, String> getWhitelabelSettings(String username) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.GET, apiUrl + "/whitelabel/sites/" + username);	// TODO push to url builder for encoding		
		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseWhitelabelSettings(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	

	public boolean requestPasswordReset(String email, String returnUrl) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/reset/request");
		request.addBodyParameter("email", email);
		request.addBodyParameter("returnUrl", returnUrl);

		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseString(getResponseBody(response)).equals("ok");
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean confirmResetPasswordToken(String email, String token) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/reset/confirm");
		request.addBodyParameter("email", email);
		request.addBodyParameter("token", token);

		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseString(getResponseBody(response)).equals("ok");
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public boolean resetPassword(String email, String token, String newPassword) throws N0ticeException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/reset/action");
		request.addBodyParameter("email", email);
		request.addBodyParameter("token", token);
		request.addBodyParameter("newPassword", newPassword);

		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseString(getResponseBody(response)).equals("ok");
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	public Map<String, String> setWhitelabelSettings(String username, Map<String, String> settings, MediaFile header, MediaFile masthead) throws N0ticeException, JsonProcessingException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/whitelabel/sites/" + username);	// TODO push to url builder for encoding		
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);		
		addStringPart(entity, "settings", new ObjectMapper().writeValueAsString(settings));	// TODO should be a RESTful body POST
		
		if (header != null) {
			entity.addPart("headerImage", new ByteArrayBody(header.getData(), header.getFilename()));
		}
		if (masthead != null) {
			entity.addPart("mastheadImage", new ByteArrayBody(masthead.getData(), masthead.getFilename()));
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);

		final Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return searchParser.parseWhitelabelSettings(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}
	
	private void populateUpdateFields(String body, String link, MediaFile image, VideoAttachment video, MultipartEntity entity) throws N0ticeException {
		if (body != null) {
			try {
				entity.addPart("body", new StringBody(body, Charset.forName(UTF_8)));
			} catch (UnsupportedEncodingException e) {
				throw new N0ticeException();
			}
		}
		if (link != null) {
			try {
				entity.addPart("link", new StringBody(link, Charset.forName(UTF_8)));
			} catch (UnsupportedEncodingException e) {
				throw new N0ticeException();
			}
		}
		if (image != null) {
			entity.addPart("image", new ByteArrayBody(image.getData(), image.getFilename()));
		}
		if (video != null) {
			entity.addPart("video", new InputStreamBody(video.getData(), video.getFilename()));			
		}
	}
	
	private Noticeboard postNewNoticeboard(String domain, String name,
			String description, boolean moderated, 
			Date endDate, Date embargoDate,
			Set<MediaType> supportedMediaTypes, String group, MediaFile cover, boolean featured, boolean isolated)
			throws N0ticeException, MissingCredentialsExeception,
			ParsingException {
		OAuthRequest request = createOauthRequest(Verb.POST, apiUrl + "/noticeboards/new");
		MultipartEntity entity = new MultipartEntity(HttpMultipartMode.BROWSER_COMPATIBLE);
		if (domain != null) {
			addEntityPartParameter(entity, "domain", domain);
		}
		addEntityPartParameter(entity, "name", name);
		addEntityPartParameter(entity, "description", description);
		addEntityPartParameter(entity, "moderated", Boolean.toString(moderated));
		addEntityPartParameter(entity, "featured", Boolean.toString(featured));
		addEntityPartParameter(entity, "isolated", Boolean.toString(isolated));
		
		if (endDate != null) {
			addEntityPartParameter(entity, "endDate", ISODateTimeFormat.dateTimeNoMillis().print(new DateTime(endDate)));
		}
		if (embargoDate != null) {
			addEntityPartParameter(entity, "embargoDate", ISODateTimeFormat.dateTimeNoMillis().print(new DateTime(embargoDate)));
		}
		if (cover != null) {
			entity.addPart("cover", new ByteArrayBody(cover.getData(), cover.getFilename()));
		}
		if (supportedMediaTypes != null) {
			String supportedMediaTypesValue = formatSupportMediaTypes(supportedMediaTypes);
			addEntityPartParameter(entity, "supportedMediaTypes", supportedMediaTypesValue.toString());
		}
		if (group != null) {
			addEntityPartParameter(entity, "group", group);
		}
		
		request.addHeader("Content-Type", entity.getContentType().getValue());
		addMultipartEntity(request, entity);
		oauthSignRequest(request);
		
		Response response = request.send();
		
		if (response.getCode() == HTTP_OK) {
	    	return noticeboardParser.parseNoticeboardResult(getResponseBody(response));
		}
		
		handleExceptions(response);
		throw new N0ticeException(getResponseBody(response));
	}

	private String formatSupportMediaTypes(Set<MediaType> supportedMediaTypes) {
		StringBuilder supportedMediaTypesValue = new StringBuilder();
		Iterator<MediaType> supportedMediaTypesIterator = supportedMediaTypes.iterator();
		while(supportedMediaTypesIterator.hasNext()) {
			supportedMediaTypesValue.append(supportedMediaTypesIterator.next());
			if (supportedMediaTypesIterator.hasNext()) {
				supportedMediaTypesValue.append(COMMA);
			}
		}
		return supportedMediaTypesValue.toString();
	}
		
	private byte[] extractMultpartBytes(MultipartEntity entity) throws IOException {
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		entity.writeTo(byteArrayOutputStream);			
		byte[] byteArray = byteArrayOutputStream.toByteArray();
		return byteArray;
	}
	
	private void handleExceptions(Response response) throws N0ticeException {
		final String responseBody = getResponseBody(response);
		log.error("Exception during n0tice api call: " + response.getCode() + ", " + responseBody);
		if (response.getCode() == HTTP_NOT_FOUND) {
			throw new NotFoundException("Not found");
		}
		if (response.getCode() == HTTP_FORBIDDEN) {
			throw new NotAllowedException(responseBody);
		}		
		if (response.getCode() == HTTP_UNAUTHORISED) {
			throw new AuthorisationException(responseBody);
		}
		if (response.getCode() == HTTP_BAD_REQUEST) {
			throw new BadRequestException(responseBody);
		}
	}
	
	private void addBodyParameter(OAuthRequest request, String parameter, String value) {
		if (value != null) {
			request.addBodyParameter(parameter, value);
		}
	}
	
	private void addEntityPartParameter(MultipartEntity entity, String parameter, String value) {
		if (value != null) {
			try {
				entity.addPart(parameter, new StringBody(value, Charset.forName(UTF_8)));
			} catch (UnsupportedEncodingException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	private void addMultipartEntity(OAuthRequest request, MultipartEntity entity) throws N0ticeException {
		try {
			request.addPayload(extractMultpartBytes(entity));
		} catch (IOException e) {			
			e.printStackTrace();
			throw new N0ticeException();
		}
	}
	
	private void oauthSignRequest(OAuthRequest request) throws MissingCredentialsExeception {
		if (scribeAccessToken != null) {
			service.signRequest(scribeAccessToken, request);
			return;
		}
		throw new MissingCredentialsExeception();
	}

	private String sign(String effectiveUrl, String secret) throws N0ticeException {
	    SecretKeySpec key;
		try {
			key = new SecretKeySpec(secret.getBytes(UTF_8), "HmacSHA1");
			Mac mac = Mac.getInstance("HmacSHA1");
			mac.init(key);
			byte[] bytes = mac.doFinal(effectiveUrl.getBytes(UTF_8));
			return new String(Base64.encodeBase64(bytes)).replace("\r\n", "");
			
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			throw new N0ticeException();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
			throw new N0ticeException();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
			throw new N0ticeException();
		}
	}
	
	private void addStringPart(MultipartEntity entity, String parameter, String value) throws N0ticeException {
		try {
			entity.addPart(parameter, new StringBody(value, Charset.forName(UTF_8)));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			throw new N0ticeException();
		}
	}
    
    private OAuthRequest createOauthRequest(Verb verb, String url) {
        OAuthRequest request = new OAuthRequest(verb, url);
        request.setConnectTimeout(HttpFetcher.HTTP_TIMEOUT, TimeUnit.MILLISECONDS);
        request.setReadTimeout(HttpFetcher.HTTP_TIMEOUT, TimeUnit.MILLISECONDS);
        return request;
    }
        
    private String getResponseBody(Response response) {
    	InputStream inputStream = response.getStream();
    	if (inputStream == null) {
    		return null;
    	}
    	
    	try {
			final List<String> lines = IOUtils.readLines(inputStream);
			IOUtils.closeQuietly(inputStream);			
			final String body = Joiner.on("\n").join(lines);
			return body;

    	} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
    
}
