package com.n0tice.api.client.parsers;

import java.util.List;
import java.util.Map;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.n0tice.api.client.exceptions.ParsingException;
import com.n0tice.api.client.model.Content;
import com.n0tice.api.client.model.Group;
import com.n0tice.api.client.model.Notification;
import com.n0tice.api.client.model.Repost;
import com.n0tice.api.client.model.ResultSet;
import com.n0tice.api.client.model.Update;
import com.n0tice.api.client.model.Vote;

public class SearchParser {
	
	private ObjectMapper mapper;
	
	public SearchParser() {		
		mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
	}
	
	public ResultSet parseSearchResults(String json) throws ParsingException {		
		try {
			return mapper.readValue(json, ResultSet.class);
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public List<Repost> parseReposts(String json) throws ParsingException {
		try {
			return mapper.readValue(json, new TypeReference<List<Repost>>(){});
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}

	public Content parseContent(String json) throws ParsingException {
		try {			
			return mapper.readValue(json, Content.class);			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public Update parseUpdate(String json) throws ParsingException {
		try {
			return mapper.readValue(json, Update.class);			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public Group parseGroup(String json) throws ParsingException {
		try {
			return mapper.readValue(json, Group.class);	
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public List<Group> parseGroups(String json) throws ParsingException {
		try {
			return mapper.readValue(json, new TypeReference<List<Group>>(){});			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public List<Notification> parseNotifications(String json) throws ParsingException {
		try {
			return mapper.readValue(json, new TypeReference<List<Notification>>(){});			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public List<Vote> parseVotes(String json) throws ParsingException {
		try {
			return mapper.readValue(json, new TypeReference<List<Vote>>(){});			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}
	
	public Map<String, String> parseWhitelabelSettings(String json) throws ParsingException {
		try {
			return mapper.readValue(json, new TypeReference<Map<String, String>>(){});			
		} catch (Exception e) {
			throw new ParsingException(e.getMessage());
		}
	}

	public String parseString(String responseBody) throws ParsingException {
		try {
			return mapper.readValue(responseBody, String.class);
		} catch (Exception e) {
			throw new ParsingException();
		}
	}
	
}
