package com.n0tice.api.client.urls;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

public class UrlBuilder {

	private static final String HISTORY = "history";
	private static final String CLOSE = "close";
	private static final String UTF_8 = "UTF-8";
	
	final private String apiUrl;
	
	public UrlBuilder(String apiUrl) {
		this.apiUrl = apiUrl;
	}
	
	public String get(String id) {
		return apiUrl + "/" + id;
	}
	
	public String userProfile(String username) {
		return apiUrl + "/user/" + urlEncode(username);
	}
	
	public String userNotifications(String username) {
		return userProfile(username) + "/notifications";
	}
	
	public String noticeBoard(String noticeboard) {
		return apiUrl + "/noticeboard/" + urlEncode(noticeboard);
	}
	
	public String userFollowedUsers(String username) {
		return userProfile(username) + "/following/users";
	}
	
	public String userFollowedNoticeboards(String username) {
		return userProfile(username) + "/following/noticeboards";
	}

	public String userNoticeboards(String username) {
		return userProfile(username) + "/noticeboards";
	}

	public String closeNoticeboard(String domain) {
		return noticeBoard(domain) + "/" + CLOSE;
	}
	
	public String getHistory(String id) {
		return get(id) + "/" + HISTORY;
	}
	
	public String moderationActions(String username) {
		return userProfile(username) + "/moderation/actions";
	}
	
	public String moderationStates(String username) {
		return userProfile(username) + "/moderation/states";
	}
	
	public String closeModerationComplaint(String contentId, int flagId) {
		return apiUrl + "/" + contentId + "/flag/" + flagId + "/close";
	}
	
	public String flagTypes() {
		return apiUrl + "/moderation/flags/types";
	}
		
	private String urlEncode(String value) {
		try {
			value = URLEncoder.encode(value, UTF_8);	// TODO incorrect encoding class
			return value.replace("+", "%20");			
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	
}
